﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using System.Collections.ObjectModel;
using System.Windows.Controls.Primitives;
using System.Windows.Markup;
using System.ComponentModel;

namespace WPF_Tests
{
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>
	public partial class Window1 : Window
	{
		static Window1()
		{
			EventManager.RegisterClassHandler(typeof(MenuItem),
				MenuItem.ClickEvent,
				new RoutedEventHandler(OnContextMenuClick));

			EventManager.RegisterClassHandler(typeof(RadTreeViewItem),
				Mouse.PreviewMouseDownEvent,
				new MouseButtonEventHandler(OnPreviewMouseDown));
		}

		public Window1()
		{
			InitializeComponent();

			treeView.ItemsSource = new ObservableCollection<OrganisationModel>(){
				OrganisationModel.GetTopOrganisation("A")
			};

			treeView.ItemPrepared += new EventHandler<RadTreeViewItemPreparedEventArgs>(treeView_ItemPrepared);
			treeView.Edited += new RadTreeViewItemEditedEventHandler(treeView_Edited);
		}

		void treeView_Edited(object sender, RadTreeViewItemEditedEventArgs e)
		{
			var treeViewItem = e.OriginalSource as RadTreeViewItem;
			var item = treeViewItem.DataContext as OrganisationModel;
			item.OrganisationName = e.NewText;
		}

		void treeView_ItemPrepared(object sender, RadTreeViewItemPreparedEventArgs e)
		{
			var treeViewItem = e.PreparedItem;
			var item = treeViewItem.DataContext as OrganisationModel;

			// You may want to mark the newly added item in a different way.
			if (item.OrganisationName.Equals("New Organisation"))
			{
				treeViewItem.Dispatcher.BeginInvoke(new Action(() =>
				{
					treeViewItem.BeginEdit();
					treeViewItem.Focus();
				}),
				System.Windows.Threading.DispatcherPriority.ContextIdle);
			}
		}

		private static void OnContextMenuClick(object sender, RoutedEventArgs e)
		{
			var menuItem = sender as MenuItem;
			var context = menuItem.DataContext as OrganisationModel;

			// Note that you may prefer to use commans, for implicity I just check
			// and switch on he header

			if ("New".Equals(menuItem.Header))
			{
				context.Children.Insert(0, new OrganisationModel() { OrganisationName = "New Organisation" });
				context.IsExpanded = true;
			}
		}

		private static void OnPreviewMouseDown(object sender, MouseEventArgs e)
		{
			if (e.RightButton == MouseButtonState.Pressed)
			{
				//the sender is the TreeViewItem:   
				var treeViewItem = sender as RadTreeViewItem;

				if (treeViewItem != null)
				{
					treeViewItem.IsSelected = true;
				}
			}
		}
	}

	public class OrganisationModel : INotifyPropertyChanged
	{
		public static OrganisationModel GetTopOrganisation(string name)
		{
			return new OrganisationModel()
			{
				OrganisationName = String.Format("Top Organisation {0}", name),
				Children = new ObservableCollection<object>()
				{ 
					new OrganisationModel()
					{ 
						OrganisationName = String.Format("Organisation 1.1 {0}",name),
						Children = new ObservableCollection<object>()
						{
							new OrganisationModel()
							{
								OrganisationName = String.Format("Organisation 2.1 {0}",name),
								Children = new ObservableCollection<object>()
								{
									new OrganisationModel()
									{
										OrganisationName = String.Format("Organisation 3.1 {0}",name)
									},
									new OrganisationModel()
									{
										OrganisationName = String.Format("Organisation 3.2 {0}",name)
									}
								}
							},
							new OrganisationModel()
							{
									OrganisationName = String.Format("Organisation 2.2 {0}",name)
							}
						}
					}
				}
			};
		}

		public OrganisationModel()
		{
			Children = new ObservableCollection<Object>();
		}

		public IList<Object> Children { get; set; }



		private String organisationName;
		/// <summary>
		/// Gets or sets .
		/// </summary>
		public String OrganisationName
		{
			get
			{
				return this.organisationName;
			}
			set
			{
				if (this.organisationName != value)
				{
					this.organisationName = value;
					OnPropertyChanged("OrganisationName");
				}
			}
		}



		private bool isExpanded;
		/// <summary>
		/// Gets or sets        .
		/// </summary>
		public bool IsExpanded
		{
			get
			{
				return this.isExpanded;
			}
			set
			{
				if (this.isExpanded != value)
				{
					this.isExpanded = value;
					OnPropertyChanged("IsExpanded");
				}
			}
		}



		public override string ToString()
		{
			return OrganisationName;
		}



		/// <summary>
		///     Called when the value of a property changes.
		/// </summary>
		/// <param name="propertyName">The name of the property that has changed.</param>
		protected virtual void OnPropertyChanged(String propertyName)
		{
			if (String.IsNullOrEmpty(propertyName))
			{
				return;
			}
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		/// <summary>
		///     Raised when the value of one of the properties changes.
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;


	}
}
